#! /usr/bin/env python

#   Copyright 2010-2011 Guy Rutenberg

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.


import portage
import logging
import subprocess
from optparse import OptionParser

class MainApp:
    def __init__(self,args=None):
        usage = "Usage: %prog [options]"
        version = "0.2"
        parser = OptionParser(usage,version=version)
        parser.add_option("","--kde", dest="kde",default=False,
                          action="store_true",
                          help="Post a notification with the number of update")
        parser.add_option("-q","--quiet", dest="quiet",default=False,
                          action="store_true",
                          help="Make as little output as possible")
        parser.add_option("-v","--verbose", dest="verbose",default=False,
                          action="store_true",
                          help="Output more information about each package")
        (self.options, self.args) = parser.parse_args(args=args)
    
    def run(self):
        self.updatable = set()
        for atom in self.get_world_entries():
            if not self.get_best_version(atom):
                print "%s: no best version" % atom
            elif self.get_best_version(atom) not in self.get_installed_versions(atom):
                self.updatable.add(atom)
                if self.options.verbose:
                    version_list = [x.replace(atom+"-",'')
                        for x in self.get_installed_versions(atom)]
                    print "%s [%s]" % (
                        self.get_best_version(atom),
                        ', '.join(version_list))
                else:
                    print atom
        if self.options.kde:
            self.notify_kde()
        if not self.options.quiet:
            print "Total:",len(self.updatable)

    def notify_kde(self):
        if len(self.updatable):
            text = "There are %i updates" % len(self.updatable)
        else:
            text = "There are no updates"
        subprocess.call(["kdialog", "--passivepopup", text, "--title", "Portage Updates"])
        
    @staticmethod
    def get_world_entries():
        """
        parses the world file and returns a set of all the atoms
        """
        f = open("/"+portage.WORLD_FILE, "r")
        atoms = set()
        for line in f.readlines():
            atom = line[:-1]
            if portage.isvalidatom(line[:-1]):
                atoms.add(atom)
        f.close()
        return atoms
    
    @staticmethod
    def get_installed_versions(atom):
        return portage.db['/']['vartree'].dbapi.match(atom)
    
    @staticmethod
    def get_best_version(atom):
        """
        Returns the best available version taking into account masking and
        keywords
        """
        available = portage.db['/']['porttree'].dbapi.match(atom)
        return portage.best(available)


if __name__ == "__main__":
    app = MainApp()
    app.run()


# vim: filetype=python ai ts=5 sts=4 et sw=4
